﻿
Class cAudioIn

    Private mRecorder As WaveLib.WaveInRecorder
    Private mRecBuffer As Int16()
    Private mWaitingZeroLevel As Boolean
    Private mDeadTimeCounter As Int32
    Private mDeadTime_uS As Single
    Private mDeadTimeSamples As Int32
    Private mTriggerLevel_Samples As Int32
    Private mSamplesPerSec As Int32
    Private mInputDevice As Int32
    Private mBufferlen As Int32
    Private mBaseLine_Enable As Boolean
    Private mBaseLinePosition_Samples As Int32
    Private mBaseLineSize_Samples As Int32
    Private mBaseLineZero As Int32

    Private mCBufferSize As Int32 = 4096
    Private mCBuffer(mCBufferSize - 1) As Int32
    Private mCBufferWriteIndex As Int32
    Private mCBufferReadIndex As Int32

    Private mRejectionArea_Samples As Int32 = 60

    Private mPBox As PictureBox
    Private mGraphics As Graphics

    Friend AudioGain As Single
    Friend AudioZero As Int32
    Friend BinMultiplier As Single
    Friend TotalPulses As Int32
    Friend Counter As Int32
    Friend PeakMeter_Positive As Single
    Friend PeakMeter_Negative As Single
    Friend NegPulses As Boolean
    Friend Enabled As Boolean = True
    Friend Bins(MAX_BIN_INDEX) As Single
    Friend TimeMicrosec As Double


    Friend Sub ResetBaseLineZero()
        mBaseLineZero = 0
    End Sub

    Friend WriteOnly Property Picture_Box() As PictureBox
        Set(ByVal value As PictureBox)
            mPBox = value
            mGraphics = Graphics.FromImage(mPBox.Image)
        End Set
    End Property

    Friend Property DeadTime_uS() As Single
        Get
            Return mDeadTime_uS
        End Get
        Set(ByVal value As Single)
            mDeadTime_uS = value
            mDeadTimeSamples = CInt(mDeadTime_uS * mSamplesPerSec / 1000000.0F)
        End Set
    End Property

    Friend Property SamplesPerSec() As Int32
        Get
            Return mSamplesPerSec
        End Get
        Set(ByVal value As Int32)
            mSamplesPerSec = value
            mDeadTimeSamples = CInt(mDeadTime_uS * mSamplesPerSec / 1000000.0F)
        End Set
    End Property

    Friend WriteOnly Property BLR_Enable() As Boolean
        Set(ByVal value As Boolean)
            mBaseLine_Enable = value
            mBaseLineZero = 0
        End Set
    End Property
    Friend WriteOnly Property BLR_Position_uS() As Single
        Set(ByVal value As Single)
            mBaseLinePosition_Samples = CInt(value * mSamplesPerSec / 1000000.0F)
        End Set
    End Property
    Friend WriteOnly Property BLR_Size_uS() As Single
        Set(ByVal value As Single)
            mBaseLineSize_Samples = CInt(value * mSamplesPerSec / 1000000.0F)
        End Set
    End Property

    Friend Property InputDevice() As Int32
        Get
            Return mInputDevice
        End Get
        Set(ByVal value As Int32)
            mInputDevice = value
        End Set
    End Property

    Friend Function RecordStart() As Boolean
        RecordStop()
        ' ------------------------------------------------- bufferLen = 20 mS
        Select Case mSamplesPerSec
            Case 44100
                mBufferlen = 940
            Case 48000
                mBufferlen = 1024
            Case 96000
                mBufferlen = 2048
            Case 144000
                mBufferlen = 3072
            Case 176400
                mBufferlen = 3760
            Case 192000
                mBufferlen = 4096
            Case 384000
                mBufferlen = 8192
            Case 768000
                mBufferlen = 16384
                'mSamplesPerSec = 201000 ' <<< error tester
            Case Else
                MsgBox("Invalid AudioIn sampling frequency")
        End Select
        Dim fmt As New WaveLib.WaveFormat(mSamplesPerSec, 16, 1)
        mRecorder = New WaveLib.WaveInRecorder(InputDevice, _
                                               fmt, _
                                               mBufferlen, _
                                               6, _
                                               New WaveLib.BufferDoneEventHandler(AddressOf DataArrived))
        If mRecorder.AudioWorking Then
            Return True
        Else
            Return False
        End If
    End Function

    Friend Sub RecordStop()
        If mRecorder IsNot Nothing Then
            Try
                mRecorder.Dispose()
            Finally
                mRecorder = Nothing
            End Try
        End If
    End Sub


    Private PreviousValue As Int32 = 0
    Private Sub DataArrived(ByVal data As IntPtr, ByVal size As Int32)
        '
        If Not Enabled Then Return
        Dim sw1 As Diagnostics.Stopwatch = New Diagnostics.Stopwatch : sw1.Start()

        ' MOD 2
        ' ===================================================
        mRejectionArea_Samples = mDeadTimeSamples
        ' ===================================================

        '
        ' -------------------------------------------------------------------- correct for mono and 16 bit samples
        size = size \ 2
        ' -------------------------------------------------------------------- redim RecBuffer if needed
        If mRecBuffer Is Nothing OrElse mRecBuffer.Length <> size Then
            ReDim mRecBuffer(size - 1)
        End If
        ' -------------------------------------------------------------------- fill RecBuffer 
        System.Runtime.InteropServices.Marshal.Copy(data, _
                                                    mRecBuffer, _
                                                    0, _
                                                    size)
        ' -------------------------------------------------------------------- initialize vars
        Dim MaxValue As Int32 = -999999
        Dim MinValue As Int32 = 999999

        Dim v As Int32
        Dim index As Int32
        Dim ValidPulse As Boolean

        ' -------------------------------------------------------------------- scan the buffer data
        For i As Int32 = 0 To mRecBuffer.Length - 1
            ' ---------------------------------------------------------------- invert pulses
            If NegPulses Then
                v = -CInt((mRecBuffer(i) - AudioZero) * AudioGain)
            Else
                v = CInt((mRecBuffer(i) + AudioZero) * AudioGain)
            End If
            ' ---------------------------------------------------------------- peak values MIN and MAX
            If v > MaxValue Then MaxValue = v
            If v < MinValue Then MinValue = v

            ' ---------------------------------------------------------------- update the CircularBuffer
            mCBufferWriteIndex += 1
            If mCBufferWriteIndex >= mCBufferSize Then mCBufferWriteIndex = 0
            mCBufferReadIndex = mCBufferWriteIndex - 500
            If mCBufferReadIndex < 0 Then mCBufferReadIndex += mCBufferSize
            mCBuffer(mCBufferWriteIndex) = v
            v = mCBuffer(mCBufferReadIndex)
            ' ----------------------------------------------------------------
            ValidPulse = True
            ' ---------------------------------------------------------------- wait the dead time
            If mDeadTimeCounter > 0 Then
                mDeadTimeCounter -= 1
            Else
                Dim mTriggerLevelFixed_Samples As Int32 = mTriggerLevel_Samples
                mTriggerLevelFixed_Samples = 10 ' 10 OK 
                ' ------------------------------------------------------------ wait Zero-level
                If mWaitingZeroLevel Then
                    If v < mTriggerLevelFixed_Samples Then
                        mWaitingZeroLevel = False
                    End If
                Else
                    ' -------------------------------------------------------- if more then trigger 
                    If v - mBaseLineZero > mTriggerLevelFixed_Samples Then
                        ' ---------------------------------------------------- if level is decreasing
                        If v < PreviousValue Then
                            ' ------------------------------------------------ test the preceding and following areas

                            ' ---------------------------------------------------------- (original)
                            index = mCBufferReadIndex - mRejectionArea_Samples \ 2
                            If index < 0 Then index += mCBufferSize
                            For j As Int32 = 1 To mRejectionArea_Samples
                                index += 1
                                If index >= mCBufferSize Then index = 0
                                ' -------------------------------------------- test the area
                                If Math.Abs(mCBuffer(index)) > PreviousValue Then
                                    ValidPulse = False
                                    Exit For
                                End If
                            Next

                            ' ---------------------------------------------------------- (mod 1)
                            'index = mCBufferReadIndex - mRejectionArea_Samples '\ 2
                            'If index < 0 Then index += mCBufferSize
                            'For j As Int32 = 1 To mRejectionArea_Samples * 2
                            '    index += 1
                            '    If index >= mCBufferSize Then index = 0
                            '    ' -------------------------------------------- test the area
                            '    If Math.Abs(mCBuffer(index)) > PreviousValue Then
                            '        ValidPulse = False
                            '        Exit For
                            '    End If
                            'Next

                            ' ---------------------------------------------------------- (mod 2)
                            'index = mCBufferReadIndex - mRejectionArea_Samples '\ 2
                            'If index < 0 Then index += mCBufferSize
                            'For j As Int32 = 1 To mRejectionArea_Samples * 2
                            '    index += 1
                            '    If index >= mCBufferSize Then index = 0
                            '    ' -------------------------------------------- test the area
                            '    If Math.Abs(mCBuffer(index) - mBaseLineZero) > PreviousValue - mBaseLineZero Then
                            '        ValidPulse = False
                            '        Exit For
                            '    End If
                            'Next

                            '
                            ' ------------------------------------------------ invalid pulse detected
                            If Not ValidPulse Then
                                PreviousValue = v
                                Continue For
                            End If
                            ' ------------------------------------------------ now a valid pulse is found
                            '   
                            ' ------------------------------------------------ Baseline restoring
                            If mBaseLine_Enable Then
                                Dim v2 As Int32
                                Dim vsum As Int32 = 0
                                Dim vmin As Int32 = 999999
                                Dim vmax As Int32 = -999999
                                index = mCBufferReadIndex - mBaseLinePosition_Samples - mBaseLineSize_Samples
                                If index < 0 Then index += mCBufferSize
                                For j As Int32 = 1 To mBaseLineSize_Samples
                                    index += 1
                                    If index >= mCBufferSize Then index = 0
                                    v2 = mCBuffer(index)
                                    If v2 > vmax Then vmax = v2
                                    If v2 < vmin Then vmin = v2
                                    vsum += v2
                                Next
                                ' -------------------------------------------- max noise allowed 32000 = 1 Volt
                                If (vmax - vmin) < 10000 Then
                                    mBaseLineZero = vsum \ mBaseLineSize_Samples
                                    If mBaseLineZero > 10000 Then mBaseLineZero = 0
                                    If mBaseLineZero < -10000 Then mBaseLineZero = 0
                                Else
                                    ValidPulse = False
                                End If
                                ' -------------------------------------------- max noise allowed = fraction of pulse
                                'If (vmax - vmin) < 0.05 * (PreviousValue - mBaseLineZero) Then
                                '    mBaseLineZero = vsum \ mBaseLineSize_Samples
                                '    If mBaseLineZero > 10000 Then mBaseLineZero = 0
                                '    If mBaseLineZero < -10000 Then mBaseLineZero = 0
                                'Else
                                '    ValidPulse = False
                                'End If
                            Else
                                mBaseLineZero = 0
                            End If

                            ' ------------------------------------------------ invalid pulse detected
                            If Not ValidPulse Then
                                PreviousValue = v
                                Continue For
                            End If


                            ' ------------------------------------------------ select the bin
                            index = CInt((PreviousValue - mBaseLineZero) * BinMultiplier)


                            ' ------------------------------------------------ copy Pulse Samples to Scope Samples
                            If mGetScopeSamples AndAlso _
                                                index >= mMinEnergyBin AndAlso _
                                                index < mMaxEnergyBin Then
                                '
                                Dim i2 As Int32 = mCBufferReadIndex - MAX_SCOPE_SAMPLES \ 2 - 1
                                If i2 < 0 Then i2 += mCBufferSize
                                For j As Int32 = 0 To MAX_SCOPE_SAMPLES - 1
                                    mScopeSamples(j) = mCBuffer(i2) - mBaseLineZero
                                    i2 += 1
                                    If i2 >= mCBufferSize Then i2 = 0
                                Next
                                mGetScopeSamples = False
                                mValidScopeSamples = True
                                mRefreshPulsePicture = True
                                ' ----------------------------------------------------------------------------
                                ' test for PreviousValue validity
                                ' ----------------------------------------------------------------------------
                                'If mScopeSamples(MAX_SCOPE_SAMPLES \ 2) <> PreviousValue - mBaseLineZero Then
                                '    Beep()
                                'End If
                                ' ----------------------------------------------------------------------------
                            End If
                            ' ------------------------------------------------ increase bin if not out of range
                            If index > 0 And index <= MAX_BIN_INDEX Then
                                Bins(index) += 1
                                TotalPulses += 1
                            End If
                            If index >= Spectrometer.mFirstBinIndex Then
                                Counter += 1
                                If Counter > 65535 Then Counter = 0
                            End If
                            ' ------------------------------------------------ initialze dead time and wait
                            mDeadTimeCounter = mDeadTimeSamples
                            mWaitingZeroLevel = True
                        End If
                    End If
                End If
            End If
            PreviousValue = v
        Next
        ' ---------------------------------------------------------------- PeakMeter
        If MaxValue > PeakMeter_Positive Then
            PeakMeter_Positive = MaxValue
        End If
        If MinValue < PeakMeter_Negative Then
            PeakMeter_Negative = MinValue
        End If

        TimeMicrosec = sw1.Elapsed.TotalMilliseconds * 1000
    End Sub



    ' =============================================================================================
    '   SCOPE
    ' =============================================================================================
    Private Const MAX_SCOPE_SAMPLES As Int32 = 400
    Private mGetScopeSamples As Boolean = False
    Private mValidScopeSamples As Boolean = False
    Private mScopeSamples(MAX_SCOPE_SAMPLES) As Int32

    Private mMaxEnergyBin As Int32
    Private mMinEnergyBin As Int32
    Private mSizeX As Int32
    Private mPosY As Int32
    Private mSizeY As Int32
    Private mNormalized As Boolean
    Private mRefreshPulsePicture As Boolean

    Friend WriteOnly Property MaxEnergy() As Single
        Set(ByVal value As Single)
            mMaxEnergyBin = Spectrometer.EnergyToBin(value)
        End Set
    End Property
    Friend WriteOnly Property MinEnergy() As Single
        Set(ByVal value As Single)
            mMinEnergyBin = Spectrometer.EnergyToBin(value)
        End Set
    End Property
    Friend WriteOnly Property SizeX() As Int32
        Set(ByVal value As Int32)
            mSizeX = value \ 2
            mRefreshPulsePicture = True
        End Set
    End Property
    Friend WriteOnly Property PosY() As Int32
        Set(ByVal value As Int32)
            mPosY = value
            mRefreshPulsePicture = True
        End Set
    End Property
    Friend WriteOnly Property SizeY() As Int32
        Set(ByVal value As Int32)
            mSizeY = value
            mRefreshPulsePicture = True
        End Set
    End Property
    Friend WriteOnly Property Normalized() As Boolean
        Set(ByVal value As Boolean)
            mNormalized = value
            mRefreshPulsePicture = True
        End Set
    End Property
    Friend Sub InvalidatePulsePicture()
        mRefreshPulsePicture = True
    End Sub
    Friend Sub Scope_Clear()
        If mGraphics Is Nothing Then Return
        mGraphics.Clear(Color.AliceBlue)
        mPBox.Invalidate()
        mGetScopeSamples = True
        mValidScopeSamples = False
        mRefreshPulsePicture = True
    End Sub
    Friend Sub Scope_Stop()
        mGetScopeSamples = False
    End Sub

    Friend Sub Scope_ShowPulses()
        If Form_Pulses.WindowState = FormWindowState.Minimized Then Return
        If Not Form_Pulses.Visible Then Return
        If mGraphics Is Nothing Then Return
        If Not mValidScopeSamples Then Return
        ' --------------------------------------------------------------------- 
        Dim i As Int32
        If Form_Pulses.btn_Run.Checked Or Form_Pulses.btn_OneShot.Checked Then
            If Form_Pulses.btn_OneShot.Checked Then
                Form_Pulses.btn_Run.Checked = False
                Form_Pulses.btn_OneShot.Checked = False
                Return
            Else
                mValidScopeSamples = False
                mGetScopeSamples = True
            End If
        Else
            mGetScopeSamples = False
        End If
        If Not mRefreshPulsePicture Then Return
        mRefreshPulsePicture = False
        ' --------------------------------------------------------------------- 
        Dim oldx, oldy, x, y As Single
        Dim mPBoxW As Int32 = mPBox.ClientSize.Width
        Dim mPBoxH As Int32 = mPBox.ClientSize.Height
        Dim ZeroX As Int32
        Dim ZeroY As Int32
        Dim kx, ky As Single
        ' -------------------------------------------------------------------- init sizes and positions
        If mNormalized Then
            ZeroY = CInt(mPBoxH * (1 - 0.4))
            ky = mScopeSamples(MAX_SCOPE_SAMPLES \ 2)
            If ky > 0 Then
                ky = 0.8333F * ZeroY / ky
            Else
                ky = 0.1
                'Form_Main.ToolStripButton_Run.Checked = False
                'MsgBox("zero or neg pulse (must never happens)")
                Return
            End If
            'If ky > 10 Then ky = 10
        Else
            ZeroY = CInt(mPBoxH * (1 - mPosY * 0.01))
            'ky = mSizeY / 200.0F
            ky = mSizeY * mPBoxH / 20000.0F
        End If
        ZeroX = mPBoxW \ 2
        kx = mPBoxW * 0.0005F * mSizeX
        ' --------------------------------------------------------------------- 
        Dim p As Pen = Pens.DarkGreen
        mGraphics.Clear(Color.AliceBlue)
        ' --------------------------------------------------------------------- Vertical lines
        For i = 0 To 16
            x = i * mPBoxW \ 16
            mGraphics.DrawLine(Pens.LightGray, x, 0, x, mPBoxH)
        Next
        ' --------------------------------------------------------------------- Horizz. lines
        For i = 0 To 10
            y = i * mPBoxH \ 10
            mGraphics.DrawLine(Pens.LightGray, 0, y, mPBoxW, y)
        Next
        ' --------------------------------------------------------------------- text
        Dim b As Brush = Brushes.DarkSlateBlue
        Dim f As Font = New Font("Arial", 10, FontStyle.Bold)
        Dim uSDiv As Single = mPBoxW / 16.0F * 1000000 / mSamplesPerSec / kx
        Dim mvDiv As Single = mPBoxH / 329.1F / ky
        Dim kevDiv As Single = Spectrometer.MillivoltToEnergy(mvDiv)
        mGraphics.DrawString(kevDiv.ToString("Y: 0.0") & " KeV/div", f, b, 10, 5)
        mGraphics.DrawString(mvDiv.ToString("Y: 0.0") & " mV/div", f, b, 10, 25)
        mGraphics.DrawString(uSDiv.ToString("X: 0.0") & " uS/div", f, b, 10, mPBoxH - 20)
        If mNormalized Then
            mGraphics.DrawString((kevDiv * 5).ToString("Pulse 0.0") & " KeV", f, b, mPBoxW \ 2 + 20, mPBoxH \ 10 - 8)
        End If
        ' --------------------------------------------------------------------- red lines
        mGraphics.DrawLine(Pens.Red, 0, ZeroY, mPBoxW, ZeroY)
        mGraphics.DrawLine(Pens.Red, ZeroX, 0, ZeroX, mPBoxH)
        ' --------------------------------------------------------------------- plot pulses
        Dim visibleSamples As Int32 = CInt(2000 / mSizeX)
        i = MAX_SCOPE_SAMPLES \ 2 - visibleSamples \ 2
        oldx = -1
        For j As Int32 = -visibleSamples \ 2 To visibleSamples \ 2
            x = j * kx + ZeroX
            y = ZeroY - mScopeSamples(i) * ky
            i += 1
            If oldx >= 0 Then
                mGraphics.DrawLine(p, oldx, oldy, x, y)
                mGraphics.DrawEllipse(p, x - 2, y - 2, 4, 4)
                'mGraphics.DrawEllipse(p, x - 1.5F, y - 1.5F, 3, 3)
            End If
            oldx = x
            oldy = y
        Next
        ' --------------------------------------------------------------------- BaseLine Restoring - position and size
        If mBaseLine_Enable Then
            mGraphics.DrawRectangle(Pens.Blue, _
                                    ZeroX - (mBaseLinePosition_Samples + mBaseLineSize_Samples) * kx, _
                                    ZeroY - 10, mBaseLineSize_Samples * kx, 20)
        End If
        ' --------------------------------------------------------------------- 
        mPBox.Invalidate()
    End Sub

End Class

