﻿Public Class cSpectrometer

    Friend mNumBins As Int32 = 0
    Friend SelectedBin As Int32 = -1
    Friend SelectedEnergy As Single
    Friend SnapToPeak As Boolean
    Friend mLastBinIndex As Int32
    Friend mFirstBinIndex As Int32

    Friend EnergyLinArray(MAX_BIN_INDEX) As Single
    Friend HeightLinArray(MAX_BIN_INDEX) As Single

    Friend mValues(MAX_BIN_INDEX) As Single
    Friend mMaxBinValue As Single
    Private mBinWithMaxValue As Int32
    Private mStrLastBin As String = ""

    Friend BackgroundSave As Boolean = False
    Friend UseReference(3) As Boolean
    Private mReferences(3, MAX_BIN_INDEX) As Single
    Private mRefMaxBinValue(3) As Single
    Private mReferenceValid(3) As Boolean

    Private mPBox As PictureBox
    Private mGraphics As Graphics
    Private mPBoxW As Int32
    Private mPBoxH As Int32
    Private mTopSpace As Int32 = 40
    Private mBottomSpace As Int32 = 4
    Private mBottomY As Single
    Private mHeightY As Single

    Private mFilterValue As Int32
    Private mXlog As Boolean
    Private mYlog As Boolean
    Private mXlogExponent As Single
    Private mYlogExponent As Single
    Private mThickLines As Boolean
    Private mZoom As Single = 1
    Private mTrimRight As Single = 750
    Private mMinEnergy As Int32
    Private mMaxEnergy As Int32
    Private mStepEnergy As Int32

    ' ================================================================================================
    '   PROPS
    ' ================================================================================================
    Friend Sub InitAll()
        mPBoxW = mPBox.ClientSize.Width
        mPBoxH = mPBox.ClientSize.Height
        mBottomY = mPBoxH - mBottomSpace
        mHeightY = mPBoxH - mTopSpace - mBottomSpace
        mMaxEnergy = CInt(XtoEnergy(mPBoxW))
        mFirstBinIndex = EnergyToBin(mMinEnergy)
        mMaxBinValue = 10
    End Sub

    Friend WriteOnly Property Picture_Box() As PictureBox
        Set(ByVal value As PictureBox)
            mPBox = value
            mGraphics = Graphics.FromImage(mPBox.Image)
        End Set
    End Property

    Friend WriteOnly Property Xlog() As Boolean
        Set(ByVal value As Boolean)
            mXlog = value
        End Set
    End Property

    Friend WriteOnly Property Ylog() As Boolean
        Set(ByVal value As Boolean)
            mYlog = value
        End Set
    End Property

    Friend WriteOnly Property ThickLines() As Boolean
        Set(ByVal value As Boolean)
            mThickLines = value
        End Set
    End Property

    Friend WriteOnly Property XlogExponent() As Double
        Set(ByVal value As Double)
            mXlogExponent = CSng(1 / value)
        End Set
    End Property

    Friend WriteOnly Property YlogExponent() As Double
        Set(ByVal value As Double)
            mYlogExponent = CSng(1 / value)
        End Set
    End Property

    Friend WriteOnly Property ScaleZoom() As Single
        Set(ByVal value As Single)
            mZoom = 300.0F / (610.0F - value)
        End Set
    End Property
    Friend WriteOnly Property TrimRight() As Single
        Set(ByVal value As Single)
            mTrimRight = value
        End Set
    End Property

    Friend Function SetNumBins(ByVal n As Int32) As Boolean
        If n <> mNumBins Then
            mNumBins = n
            Return True
        Else
            Return False
        End If
    End Function

    Friend Function GetNumBins() As Int32
        Return mNumBins
    End Function

    Friend Property Filter() As Int32
        Set(ByVal value As Int32)
            mFilterValue = value
        End Set
        Get
            Return mFilterValue
        End Get
    End Property

    Friend Property MinEnergy() As Int32
        Set(ByVal value As Int32)
            mMinEnergy = value
        End Set
        Get
            Return mMinEnergy
        End Get
    End Property

    Friend Sub SetSelectedBinFromX(ByVal x As Int32)
        SelectedBin = XtoBin(x)
        SelectedEnergy = XtoEnergy(x)
        Form_Main.ShowSelectedBinData()
    End Sub

    Friend Sub ChangeSelectedBin(ByVal delta As Int32)
        SelectedBin += delta
        If SelectedBin > mLastBinIndex Then SelectedBin = mLastBinIndex
        If SelectedBin < mFirstBinIndex Then SelectedBin = mFirstBinIndex
        SelectedEnergy = BinToEnergy(SelectedBin)
        Form_Main.ShowSelectedBinData()
    End Sub

    Friend Function GetValidPulses() As Int32
        GetValidPulses = AudioIn.TotalPulses
        For i As Int32 = 0 To mFirstBinIndex - 1
            GetValidPulses -= CInt(AudioIn.Bins(i))
        Next
        If GetValidPulses < 0 Then GetValidPulses = 0
    End Function


    ' ================================================================================================
    '   DRAW SPECTRUM
    ' ================================================================================================
    Friend Sub Draw()
        ' --------------------------------------------------------------------- scale coeff
        InitAll()
        ' --------------------------------------------------------------------- LastBinIndex
        mStrLastBin = ""
        Dim n As Int32 = XtoBin(mPBoxW)
        If n >= MAX_BIN_INDEX Then
            n = MAX_BIN_INDEX
            mStrLastBin = "Limited by bins - Decrease bin number"
        End If
        If n > MillivoltToBin(1000) - 1 Then
            n = CInt(MillivoltToBin(1000)) - 1
            mStrLastBin = "Limited by audio card saturation - Decrease signal level"
        End If
        mLastBinIndex = n

        ' --------------------------------------------------------------------- clear graphics
        mGraphics.Clear(Color.AliceBlue)
        ' --------------------------------------------------------------------- Mark Isotopes 
        MarkIsotopes()
        ' --------------------------------------------------------------------- scale divisions
        DrawScaleDivisions()
        ' --------------------------------------------------------------------- IIR Filter and MaxBinValue
        ApplyFilter(mFirstBinIndex, _
                    mLastBinIndex, _
                    AudioIn.Bins, _
                    mValues, _
                    mMaxBinValue, _
                    mBinWithMaxValue)
        ' --------------------------------------------------------------------- Background and References
        BackgroundAndReferences()
        ' --------------------------------------------------------------------- draw spectrum
        DrawSpectrum()
        ' --------------------------------------------------------------------- draw Selected Bin red line
        DrawSelectedBin()

        ' --------------------------------------------------------------------- final refresh
        mPBox.Invalidate()
        'mPBox.Refresh()
    End Sub




    ' =========================================================================================================
    '   Mark Isotopes
    ' =========================================================================================================
    Private Sub MarkIsotopes()
        If IdentiFinder IsNot Nothing AndAlso IdentiFinder.Enabled Then
            IdentiFinder.MarkIsotopes(mGraphics)
        Else
            MarkIsotope("x32", 32)
            MarkIsotope("Am-241", 59.5)
            MarkIsotope("U-235", 186)
            MarkIsotope("Cs-137", 660)
            MarkIsotope("K-40", 1460)
        End If
    End Sub
    Friend Sub MarkIsotope(ByVal name As String, ByVal e As Single)
        Dim f As Font = New Font("Arial", 8)
        Dim x As Single = EnergyToX(e) - 1
        mGraphics.DrawLine(New Pen(Color.Orange, 1), x, 2, x, mPBoxH)
        mGraphics.DrawString(name, f, Brushes.Black, x, 2)
    End Sub
    Friend Sub MarkPeak(ByVal text As String, ByVal e As Single)
        Dim f As Font = New Font("tahoma", 9, FontStyle.Regular)
        Dim x As Single = EnergyToX(e)
        Dim y As Single = mPBoxH - 34
        mGraphics.FillRectangle(Brushes.AliceBlue, x - 20, y + 2, 40, 24)
        'mGraphics.DrawLine(Pens.Red, x, y - 10, x, y - 5)
        mGraphics.DrawString(text, f, Brushes.Black, x - 22, y)
    End Sub


    ' =========================================================================================================
    '   Scale divisions
    ' =========================================================================================================
    Private Sub DrawScaleDivisions()
        Dim x, y As Single
        ' --------------------------------------------------------------------- vert. divisions
        Dim k As Single = mMaxEnergy / CSng(mPBoxW)
        If mXlog Then k *= CSng(3.5 * 0.2 ^ mXlogExponent)
        Select Case k
            Case Is > 6.4F : mStepEnergy = 500
            Case Is > 3.2F : mStepEnergy = 200
            Case Is > 1.6F : mStepEnergy = 100
            Case Is > 0.9F : mStepEnergy = 50
            Case Is > 0.4F : mStepEnergy = 20
            Case Is > 0.2F : mStepEnergy = 10
            Case Is > 0.1F : mStepEnergy = 5
            Case Is > 0.04F : mStepEnergy = 2
            Case Else : mStepEnergy = 1
        End Select
        Dim f As Font = New Font("Arial", 8)
        For i As Int32 = mStepEnergy To mMaxEnergy Step mStepEnergy
            x = EnergyToX(i)
            mGraphics.DrawLine(Pens.Gray, x, 20, x, mPBoxH)
            mGraphics.DrawString(i.ToString, f, Brushes.Black, x, 20)
        Next
        ' --------------------------------------------------------------------- horiz. divisions
        For i As Int32 = 0 To 23
            y = ValueToY(i / 23.0F)
            mGraphics.DrawLine(Pens.LightGray, 0, y, mPBoxW, y)
        Next
    End Sub


    ' =========================================================================================================
    '   FILTER
    ' =========================================================================================================
    Private Sub ApplyFilter(ByVal FirstBin As Int32, _
                            ByVal LastBin As Int32, _
                            ByVal ArraySource() As Single, _
                            ByRef ArrayDest() As Single, _
                            ByRef MaxBinValue As Single, _
                            ByRef BinWithMaxValue As Int32)
        ' --------------------------------------------------------------------- filter coeff
        Dim FilterCoeff As Single = 0.3F * (50000.0F / mNumBins) / (mFilterValue + 10)
        If FilterCoeff > 1 Then FilterCoeff = 1
        '---------------------------------------------------------------------- prepare values 0 to FirstBinIndex -1
        For i As Int32 = 0 To FirstBin - 1
            ArrayDest(i) = 0
        Next
        '---------------------------------------------------------------------- IIR Filter - first pass - UP
        Dim v, vnew As Single
        v = ArraySource(FirstBin)
        For i As Int32 = FirstBin To LastBin
            vnew = ArraySource(i) * HeightLinArray(i)
            ' ----------------------------------------- filter
            v += (vnew - v) * FilterCoeff
            'Dim d As Single = vnew - v
            'v += Math.Sign(d) * d * d * FilterCoeff
            ' ----------------------------------------- store filtered value
            ArrayDest(i) = v
        Next
        '---------------------------------------------------------------------- IIR Filter - second pass - DOWN
        For i As Int32 = LastBin To FirstBin Step -1
            vnew = ArraySource(i) * HeightLinArray(i)
            ' ----------------------------------------- filter
            v += (vnew - v) * FilterCoeff
            'Dim d As Single = vnew - v
            'v += Math.Sign(d) * d * d * FilterCoeff
            ' ----------------------------------------- add up and down filter passes
            ArrayDest(i) += v
            ' ----------------------------------------- update MaxBinValue
            If ArrayDest(i) > MaxBinValue Then
                MaxBinValue = ArrayDest(i)
                BinWithMaxValue = i
            End If
        Next
    End Sub

    'Private Sub ApplyFilter(ByVal ArraySource() As Single, _
    '                        ByRef ArrayDest() As Single, _
    '                        ByRef MaxBinValue As Single, _
    '                        ByRef BinWithMaxValue As Int32)
    '    '---------------------------------------------------------------------- prepare values 0 to FirstBinIndex -1
    '    For i As Int32 = 0 To mFirstBinIndex - 1
    '        ArrayDest(i) = 0
    '    Next
    '    '---------------------------------------------------------------------- FIR Filter
    '    Dim v, v2 As Single
    '    v = ArraySource(mFirstBinIndex)
    '    For i As Int32 = mFirstBinIndex To mFirstBinIndex + 4
    '        v += ArraySource(i)
    '    Next
    '    For i As Int32 = mFirstBinIndex To mLastBinIndex - 5
    '        v -= ArraySource(i)
    '        v += ArraySource(i + 5)
    '        ' -----------------------------------------
    '        v2 = v * HeightLinArray(i + 3)
    '        'v2 /= 5.0F
    '        ArrayDest(i + 3) = v2
    '        ' ----------------------------------------- update MaxBinValue
    '        If v2 > MaxBinValue Then
    '            MaxBinValue = v2
    '            BinWithMaxValue = i + 3
    '        End If
    '    Next
    'End Sub



    ' =========================================================================================================
    '   Background and References
    ' =========================================================================================================
    Private Sub BackgroundAndReferences()
        ' ------------------------------------------------------------- background
        If BackgroundSave Then
            BackgroundSave = False
            For i As Int32 = 1 To mLastBinIndex
                mReferences(0, i) = mValues(i)
            Next
            mRefMaxBinValue(0) = mMaxBinValue
        End If
        ' ------------------------------------------------------------- references
        If UseReference(1) Then
            SaveAndDrawReference(1)
        Else
            If AudioIn.Enabled Then mReferenceValid(1) = False
        End If
        If UseReference(2) Then
            SaveAndDrawReference(2)
        Else
            If AudioIn.Enabled Then mReferenceValid(2) = False
        End If
        If UseReference(3) Then
            SaveAndDrawReference(3)
        Else
            If AudioIn.Enabled Then mReferenceValid(3) = False
        End If
    End Sub
    Private Sub SaveAndDrawReference(ByVal nref As Int32)
        If Not mReferenceValid(nref) Then
            ' -------------------------------------------------------------------- save reference
            If AudioIn.Enabled Then
                For i As Int32 = 1 To mReferences.GetUpperBound(1)
                    mReferences(nref, i) = mValues(i)
                Next
            End If
            mRefMaxBinValue(nref) = mMaxBinValue
            mReferenceValid(nref) = True
        Else
            ' -------------------------------------------------------------------- draw reference
            Dim p As Pen
            Select Case nref
                Case 1 : p = New Pen(Color.FromArgb(140, 80, 0), 1)
                Case 2 : p = New Pen(Color.FromArgb(0, 140, 160), 1)
                Case Else : p = New Pen(Color.FromArgb(160, 150, 0), 1)
            End Select
            Dim x, y As Single
            Dim oldx As Single = -100
            Dim oldy As Single = mPBox.ClientSize.Height
            If mThickLines Then
                ' ----------------------------------------------------------------- draw thick line
                For i As Int32 = mFirstBinIndex To mValues.Length - 1
                    x = BinToX(i) - 0.5F
                    y = ValueToY(mReferences(nref, i) / mRefMaxBinValue(nref))
                    If oldx >= 0 Then
                        mGraphics.DrawLine(p, oldx, oldy, x, y)
                        mGraphics.DrawLine(p, oldx + 1, oldy, x + 1, y)
                    End If
                    oldx = x
                    oldy = y
                Next
            Else
                ' ----------------------------------------------------------------- draw thin line
                For i As Int32 = mFirstBinIndex To mValues.Length - 1
                    x = BinToX(i)
                    y = ValueToY(mReferences(nref, i) / mRefMaxBinValue(nref))
                    If oldx >= 0 Then
                        mGraphics.DrawLine(p, oldx, oldy, x, y)
                    End If
                    oldx = x
                    oldy = y
                Next
            End If
        End If
    End Sub


    ' =========================================================================================================
    '   Draw Spectrum
    ' =========================================================================================================
    Private Sub DrawSpectrum()
        Dim x, y, v As Single
        Dim oldx, oldy As Single
        Dim kbkg As Single = mMaxBinValue / mRefMaxBinValue(0)
        oldx = -100
        oldy = mPBoxH
        If mThickLines Then
            ' ----------------------------------------------------------------- draw thick line
            Dim p As Pen = New Pen(Color.FromArgb(40, 140, 0))
            For i As Int32 = mFirstBinIndex To mLastBinIndex
                x = BinToX(i) - 0.5F
                v = mValues(i)
                If UseReference(0) Then
                    v -= mReferences(0, i) * kbkg
                End If
                y = ValueToY(v / mMaxBinValue)
                If oldx >= 0 Then
                    mGraphics.DrawLine(p, oldx, oldy, x, y)
                    mGraphics.DrawLine(p, oldx + 1, oldy, x + 1, y)
                End If
                oldx = x
                oldy = y
            Next
        Else
            ' ----------------------------------------------------------------- draw thin line
            Dim p As Pen = New Pen(Color.FromArgb(40, 120, 0))
            For i As Int32 = mFirstBinIndex To mLastBinIndex
                x = BinToX(i)
                v = mValues(i)
                If UseReference(0) Then
                    v -= mReferences(0, i) * kbkg
                End If
                y = ValueToY(v / mMaxBinValue)
                If oldx >= 0 Then
                    mGraphics.DrawLine(p, oldx, oldy, x, y)
                    'mGraphics.DrawEllipse(p, x - 1.5F, y - 1.5F, 3, 3)
                End If
                oldx = x
                oldy = y
            Next
        End If
        ' --------------------------------------------------------------------- signal the end of the valid zone
        If mStrLastBin <> "" Then
            Dim f As Font = New Font("Arial", 11)
            Dim s As SizeF = mGraphics.MeasureString(mStrLastBin, f)
            mGraphics.FillRectangle(Brushes.AliceBlue, x - s.Width, mPBoxH - 24, s.Width, s.Height)
            mGraphics.DrawString(mStrLastBin, f, Brushes.Black, x - s.Width + 8, mPBoxH - 24)
        End If
    End Sub


    ' =========================================================================================================
    '   Selected bin
    ' =========================================================================================================
    Private Sub DrawSelectedBin()
        Dim x As Single
        If SelectedBin > 0 Then
            If SnapToPeak Then
                SelectedBin = CalcFWHM_NearTo(SelectedBin)
            End If
            x = BinToX(SelectedBin)
            If x >= 0 And x < mPBoxW Then
                mGraphics.DrawLine(Pens.OrangeRed, x, mTopSpace - 4, x, mPBoxH - If(SnapToPeak, 35, 2))
            End If
            Form_Main.ShowSelectedBinData()
        End If
        ' --------------------------------------------------------------------- FWHM
        'CalcFWHM(mBinWithMaxValue)
    End Sub
    Private Function CalcFWHM_NearTo(ByVal bin As Int32) As Int32
        Dim bin1 As Int32 = bin - 20
        Dim bin2 As Int32 = bin + 20
        If bin1 < mFirstBinIndex Then bin1 = mFirstBinIndex
        If bin2 > mLastBinIndex Then bin2 = mLastBinIndex
        Dim v As Single = 0
        For i As Int32 = bin1 To bin2
            If mValues(i) > v Then
                v = mValues(i)
                bin = i
            End If
        Next
        CalcFWHM(bin)
        Return bin
    End Function
    Private Sub CalcFWHM(ByVal bin As Int32)
        If bin >= MAX_BIN_INDEX Then Return
        Dim e As Single = BinToEnergy(bin)
        ' -------------------------------------------------- cesium only
        'If Math.Abs(e - 660) > 100 Then Return
        '
        Dim bin1, bin2 As Int32
        Dim halfvalue As Single = mValues(bin) / 2.0F
        ' -------------------------------------------------- find left bin with count = 50%
        For j As Int32 = bin To mFirstBinIndex Step -1
            If mValues(j) < halfvalue Then
                bin1 = j + 1
                Exit For
            End If
        Next
        ' -------------------------------------------------- find right bin with count = 50%
        For j As Int32 = bin To mLastBinIndex
            If mValues(j) < halfvalue Then
                bin2 = j - 1
                Exit For
            End If
        Next
        ' -------------------------------------------------- 
        If bin1 = 0 Or bin2 = 0 Then Return
        Dim e1 As Single = BinToEnergy(bin1)
        Dim e2 As Single = BinToEnergy(bin2)
        Dim fwhm As Single = (e2 - e1) / e
        MarkPeak(fwhm.ToString(" 0.0 %", Globalization.CultureInfo.InvariantCulture) & vbCrLf & "(fwhm)", e)

        ' -------------------------------------------------- trying to find the noise baseline
        'Dim midsize As Single = Math.Min(bin - bin1, bin2 - bin) * 1.4F

        'Dim b1 As Int32 = CInt(bin - midsize)
        'Dim b2 As Int32 = CInt(bin + midsize)
        'If b1 < 0 Then Return
        'Dim x1 As Single = EnergyToX(BinToEnergy(b1))
        'Dim x2 As Single = EnergyToX(BinToEnergy(b2))

        'Dim y1 As Single = ValueToY(mValues(CInt(b1)) / mMaxBinValue)
        'Dim y2 As Single = ValueToY(mValues(CInt(b2)) / mMaxBinValue)

        'mGraphics.DrawLine(Pens.Black, x1, y1, x2, y2)


        ' ------------------------------ unused
        'mGraphics.DrawRectangle(Pens.Black, x1, mPBoxH - 40, x2 - x1, 10)
        'mGraphics.DrawLine(Pens.Black, x1, mPBoxH - 40, x1, mPBoxH - 300)
        'mGraphics.DrawLine(Pens.Black, x2, mPBoxH - 40, x2, mPBoxH - 300)
    End Sub



    ' =========================================================================================================
    '   Export and Import
    ' =========================================================================================================
    Friend Function GetBinDataString(ByVal ExportWithHeader As Boolean, ByVal DecimalSeparator As String, ByVal FieldSeparator As String) As String
        Dim s As String = ""
        If ExportWithHeader Then
            s &= "-------------------------------------------" & vbCrLf
            s &= " Theremino_MCA - Pulse Height Histogram    " & vbCrLf
            s &= "-------------------------------------------" & vbCrLf
            s &= "       Started at: " & Date.Now.ToString("yyyy/MM/dd HH':'mm':'ss") & vbCrLf
            s &= "       Total time: " & TotalSeconds.ToString & " sec." & vbCrLf
            s &= "     Total pulses: " & ValidPulses.ToString & vbCrLf
            s &= "  Pulses per sec.: " & (ValidPulses / CSng(TotalSeconds)).ToString("0.0") & vbCrLf
            s &= "   Minimum energy: " & mMinEnergy.ToString() & " KeV" & vbCrLf
            s &= "        Dead time: " & AudioIn.DeadTime_uS.ToString & " uS" & vbCrLf
            s &= "         Num bins: " & (mLastBinIndex - mFirstBinIndex).ToString & vbCrLf
            s &= "         Sampling: " & AudioIn.SamplesPerSec.ToString & " Hz" & vbCrLf
            s &= "-------------------------------------------" & vbCrLf
            s &= " Energy(KeV)    Counts                     " & vbCrLf
            s &= "-------------------------------------------" & vbCrLf
        End If
        Dim s2 As String
        For i As Int32 = mFirstBinIndex To mLastBinIndex
            s2 = BinToEnergy(i).ToString(" 0.0", System.Globalization.CultureInfo.InvariantCulture)
            s2 = s2.Replace(".", DecimalSeparator)
            s &= s2 & FieldSeparator & StrDup(15 - s2.Length, " ") & AudioIn.Bins(i).ToString & vbCrLf
        Next
        Return s
    End Function

    Friend Sub FillBinsFromStringArray(ByVal destination As Int32, ByVal StringArray() As String)
        ' --------------------------------------------------- find first line
        Dim FirstLine As Int32
        For i As Int32 = 0 To StringArray.Length - 1
            If Val(StringArray(i)) > 0 Then
                FirstLine = i
                Exit For
            End If
        Next
        ' --------------------------------------------------- find separators
        Dim s As String = StringArray(FirstLine)
        Dim dotpos As Int32 = InStr(s, ".")
        Dim commapos As Int32 = InStr(s, ",")
        Dim semicolonpos As Int32 = InStr(s, ";")
        Dim DecimalSep As String
        Dim FieldSep As String
        If semicolonpos = 0 And dotpos = 0 And commapos > 0 Then
            DecimalSep = ""
            FieldSep = ","
        ElseIf semicolonpos = 0 And dotpos > 0 And commapos > dotpos Then
            DecimalSep = "."
            FieldSep = ","
        ElseIf dotpos = 0 And commapos > 0 And semicolonpos > commapos Then
            DecimalSep = ","
            FieldSep = ";"
        Else
            MsgBox("Unrecognized file separators." & vbCrLf & _
                   "Please use a dot as decimal separator and a comma as field separator" & vbCrLf & _
                   "or a comma as decimal separator and a semicolon as field separator")
            Return
        End If
        ' --------------------------------------------------- extract data
        Dim ndata As Int32 = StringArray.Length - FirstLine
        Dim en(ndata - 1) As Single
        Dim cts(ndata - 1) As Single
        Dim s2() As String
        For i As Int32 = 0 To ndata - 1
            s2 = Strings.Split(StringArray(i + FirstLine), FieldSep)
            s2(0) = s2(0).Replace(DecimalSep, ".")
            en(i) = CSng(Val(s2(0)))
            cts(i) = CSng(Val(s2(1)))
        Next
        ' --------------------------------------------------- fill bins with interpolated data
        Dim SourceIndex1 As Int32
        Dim SourceIndex2 As Int32
        Dim DestBinEnergy As Single
        Dim v As Single
        Dim ArraySource(MAX_BIN_INDEX) As Single
        Dim ArrayDest(MAX_BIN_INDEX) As Single
        For i As Int32 = 0 To MAX_BIN_INDEX
            DestBinEnergy = BinToEnergy(i)
            ' ----------------------------------------------- find index of inporting-data with more energy
            For j As Int32 = 0 To ndata - 1
                If en(j) >= DestBinEnergy Then
                    SourceIndex2 = j
                    Exit For
                End If
            Next
            If SourceIndex2 > ndata - 1 Then SourceIndex1 = ndata - 1
            SourceIndex1 = SourceIndex2 - 1
            If SourceIndex1 < 0 Then SourceIndex1 = 0

            ' ------------------------------------------- interpolate values
            v = Interpolate(en(SourceIndex1), _
                            en(SourceIndex2), _
                            DestBinEnergy, _
                            cts(SourceIndex1), _
                            cts(SourceIndex2))

            'v = cts(SourceIndex2)

            ArraySource(i) = v
        Next
        ' --------------------------------------------------- apply filter with actual FilterCoeff and HeightEqualization
        Dim MaxBinValue As Single
        Dim BinWithMaxValue As Int32
        ApplyFilter(mFirstBinIndex, _
                    MAX_BIN_INDEX, _
                    ArraySource, _
                    ArrayDest, _
                    MaxBinValue, _
                    BinWithMaxValue)
        ' --------------------------------------------------- copy the destination array
        For i As Int32 = 0 To MAX_BIN_INDEX
            mReferences(destination, i) = ArrayDest(i)
        Next
        mRefMaxBinValue(destination) = MaxBinValue
        mReferenceValid(destination) = True
        UseReference(destination) = True
    End Sub

    ' ================================================================================================
    '   Y SCALE CONVERSIONS
    ' ================================================================================================
    Private Function ValueToY(ByVal value As Single) As Single
        If value > 0 Then
            If mYlog Then
                value = CSng(value ^ mYlogExponent)
            End If
        Else
            value = 0
        End If
        Return mBottomY - value * mHeightY
    End Function


    ' ================================================================================================
    '   X SCALE CONVERSIONS
    ' ================================================================================================
    Private Function x_to_exp(ByVal x As Single) As Single
        If x < 0 Then Return x
        x /= mPBoxW
        x = CSng(x ^ (1.0 / mXlogExponent))
        x *= mPBoxW
        Return x
    End Function
    Private Function x_to_log(ByVal x As Single) As Single
        If x < 0 Then Return x
        x /= mPBoxW
        x = CSng(x ^ mXlogExponent)
        x *= mPBoxW
        Return x
    End Function

    '  XSCALE <--> ENERGY
    ' =============================================================================== original
    Friend Function XtoEnergy(ByVal x As Single) As Single
        If mXlog Then
            x = x_to_exp(x)
        End If
        Return x * 1600.0F / mPBoxW / mZoom
    End Function
    Friend Function EnergyToX(ByVal e As Single) As Single
        e = e * mPBoxW / 1600.0F * mZoom
        If mXlog Then
            e = x_to_log(e)
        End If
        Return e
    End Function

    '  XSCALE <--> BINS
    ' =============================================================================== simplified (without mZoom)
    Friend Function XtoBin(ByVal x As Single) As Int32
        Return EnergyToBin(XtoEnergy(x))
    End Function
    Friend Function BinToX(ByVal bin As Int32) As Single
        Return EnergyToX(BinToEnergy(bin))
    End Function


    '  ENERGY <--> BINS
    ' =============================================================================== WITHOUT LINEARIZER
    'Friend Function EnergyToBin(ByVal e As Single) As Int32
    '    Return CInt(e / 8.0F * mNumBins / mTrimRight)
    'End Function
    'Friend Function BinToEnergy(ByVal b As Int32) As Single
    '    Return b * 8 * mTrimRight / mNumBins
    'End Function
    'Friend Function BinToEnergy(ByVal b As Single) As Single
    '    Return b * 8 * mTrimRight / mNumBins
    'End Function
    ' =============================================================================== WITH LINEARIZER
    Friend Function EnergyToBin(ByVal e As Single) As Int32
        ' TODO ---------------------------------------------------- use a more efficient lookup-table
        For bin As Int32 = 0 To MAX_BIN_INDEX
            If BinToEnergy(bin) >= e Then
                Return bin
            End If
        Next
        Return MAX_BIN_INDEX
    End Function
    Friend Function BinToEnergy(ByVal b As Int32) As Single
        Return 8 * Spectrometer.EnergyLinArray(b) / mNumBins
    End Function
    Friend Function BinToEnergy(ByVal b As Single) As Single
        ' -----------------------------------------------------------
        '  BinToEnergy interpolated 
        '  used only by "MillivoltToEnergy" to show pulse energies
        ' -----------------------------------------------------------
        Dim e As Single
        Dim bin As Int32 = CInt(Math.Floor(b))
        If bin > MAX_BIN_INDEX - 1 Then
            e = Spectrometer.EnergyLinArray(CInt(b))
        Else
            e = Interpolate(bin, bin + 1, b, Spectrometer.EnergyLinArray(bin), Spectrometer.EnergyLinArray(bin + 1))
        End If
        Return 8 * e / mNumBins
    End Function


    '  MILLIVOLT <--> BINS
    ' ===============================================================================
    Friend Function MillivoltToBin(ByVal mv As Single) As Single
        Return mv * AudioIn.BinMultiplier * 32768.0F / 1000.0F
    End Function

    '  MILLIVOLT <--> ENERGY
    ' ===============================================================================
    Friend Function MillivoltToEnergy(ByVal mv As Single) As Single
        Return BinToEnergy(MillivoltToBin(mv))
    End Function


End Class
