﻿
Imports System.Math
Imports System.Drawing.Imaging

Module Module_SaveLoad

    Friend EventsAreEnabled As Boolean = False


    ' =======================================================================================================
    '   APP TITLE AND VERSION
    ' =======================================================================================================
    Friend Function AppTitleAndVersion(Optional ByVal Title As String = "") As String
        If Title = "" Then Title = Replace(My.Application.Info.AssemblyName, "_", " ")
        Dim s() As String = Split(My.Application.Info.Version.ToString, ".")
        Return Title & " - V" & s(0) & "." & s(1)
    End Function


    ' =======================================================================================================
    '   Files
    ' =======================================================================================================
    ' returns lower-case extension with initial dot
    Public Function GetExtension(ByVal str As String) As String
        On Error Resume Next
        Return LCase(IO.Path.GetExtension(str))
    End Function
    Public Function RemoveExtension(ByVal str As String) As String
        On Error Resume Next
        Return PlatformAdjustedFileName(IO.Path.GetDirectoryName(str) & "\" & IO.Path.GetFileNameWithoutExtension(str))
    End Function
    Public Function FileExists(ByVal fileName As String) As Boolean
        Return My.Computer.FileSystem.FileExists(fileName)
    End Function
    Public Function FolderExists(ByVal FolderName As String) As Boolean
        If FolderName.Length < 2 Then Return False
        FolderName = LCase(FolderName)
        Select Case FolderName
            Case "a:\", "b:\", "c:\", "d:\", "e:\", "f:\", "g:\", "h:\", "i:\", "j:\", "k:\"
                Return True
        End Select
        Return My.Computer.FileSystem.DirectoryExists(FolderName)
    End Function


    ' =======================================================================================================
    '   Platform selection
    ' =======================================================================================================
    'Friend CanUseWindowsDlls As Boolean = _CanUseWindowsDlls()
    Friend Function CanUseWindowsDlls() As Boolean
        Select Case Environment.OSVersion.Platform
            Case PlatformID.Win32NT, PlatformID.Win32S, PlatformID.Win32Windows, PlatformID.WinCE
                Return True
            Case Else
                Return False
        End Select
    End Function
    Friend Function PlatformAdjustedFileName(ByVal FileName As String, _
                                             Optional ByVal DefaultName As String = "") As String
        If CanUseWindowsDlls Then
            FileName = FileName.Replace("/", "\")
            Return FileName
        Else
            If FileName.Contains(":") Then FileName = DefaultName
            FileName = FileName.Replace("\", "/")
            Return FileName
        End If
    End Function

    ' =======================================================================================
    '  FORM
    ' =======================================================================================

    Friend Sub LimitFormPosition(ByVal f As System.Windows.Forms.Form)
        If f.WindowState <> FormWindowState.Normal Then Return
        GetMaxScreenBounds()
        EnsureFormVisible(f)
    End Sub

    Private sw As Int32
    Private sh As Int32
    Private Sub GetMaxScreenBounds()
        ' -------------------------------------- Gets an array of all the screens connected to the system.
        Dim Screens() As System.Windows.Forms.Screen = System.Windows.Forms.Screen.AllScreens
        ' -------------------------------------- For each screen
        Dim Index As Int32
        For Index = 0 To Screens.GetUpperBound(0)
            sw = Max(sw, Screens(Index).Bounds.Left + Screens(Index).Bounds.Width)
            sh = Max(sh, Screens(Index).Bounds.Top + Screens(Index).Bounds.Height)
        Next
    End Sub

    Private Sub EnsureFormVisible(ByRef frm As Form)
        With frm
            .Width = Math.Min(.Width, sw)               ' non oltre le dimensioni del VIRTUALSCREEN
            .Height = Math.Min(.Height, sh)
            .Width = Math.Max(.Width, 32)               ' non meno di 32x24
            .Height = Math.Max(.Height, 24)             ' non meno di 32x24
            .Left = Math.Min(.Left, sw - 50)            ' non oltre il bordo destro - 50
            .Top = Math.Min(.Top, sh - 50)              ' non oltre il bordo inferiore - 50
            .Left = Math.Max(.Left, 90 - .Width)        ' non meno del bordo sinistro + 90
            .Top = Math.Max(.Top, -20)                  ' non sopra al bordo superiore
        End With
    End Sub


    ' (The value of the RestoreBounds property is valid only 
    '   when the WindowState property of the Form class is not equal to Normal)
    Friend Function GetFormRectangle(ByVal frm As Form) As Rectangle
        Dim r As Rectangle
        If frm.WindowState = FormWindowState.Normal And frm.Bounds.Left > -1000 Then
            r = frm.Bounds
        Else
            r = frm.RestoreBounds
        End If
        Return r
    End Function


    ' ================================================================================================
    '  Private Read-Write functions
    ' ================================================================================================

    Private Function TabString(ByVal Name As String, _
                              Optional ByVal Value As Double = Double.NaN, _
                              Optional ByVal fmt As String = "") As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)
        If Double.IsNaN(Value) Then
            Return Name
        Else
            Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString(fmt)
        End If
    End Function

    Private Function TabString(ByVal Name As String, _
                                  ByVal Value As Boolean) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString
    End Function

    Private Function TabString(ByVal Name As String, _
                                ByVal Value As String) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value
    End Function

    Private Function Val_Double(ByVal l As String) As Double
        Return Val(Replace(l, ",", "."))
    End Function

    Private Function MultilineToString(ByVal Name As String, _
                                  ByVal Value As String) As String

        Value = Value.Replace(vbCrLf, "%M")
        Return Name & "=" & Value
    End Function

    Private Function StringToMultiline(ByVal Value As String) As String
        Value = Value.Replace("%M", vbCrLf)
        Return Value
    End Function


    Friend Function ExtractParamName(ByRef s As String) As String
        ' ------------------------- Returns the first field from begin to the first "=" symbol
        ' -------------------------  and removes it from the string
        Dim i As Int32
        i = InStr(s, "=")
        If i > 0 Then
            ExtractParamName = Trim(Strings.Left(s, i - 1))
            s = Trim(Mid(s, i + 1))
        Else
            ExtractParamName = Trim(s)
            s = ""
        End If
    End Function

    Private Function AssemblyName() As String
        Return System.Reflection.Assembly.GetExecutingAssembly.GetName.Name
    End Function



    ' ==================================================================================================
    '  SAVE LOAD -- Program INI
    ' ==================================================================================================
    Friend Sub SaveConfigurationAs()
        Dim sfd As SaveFileDialog = New SaveFileDialog()
        sfd.DefaultExt = ".txt"
        sfd.Filter = "Configuration file (*.txt)|*txt"
        'sfd.FileName = AssemblyName() & "_INI_" & Date.Now.ToString("yyyy_MM_dd_HH_mm_ss")
        sfd.FileName = "MCA_INI_" & Date.Now.ToString("yyyy_MM_dd_HH_mm_ss")
        If sfd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            Save_INI(sfd.FileName)
        End If
    End Sub

    Friend Sub LoadConfiguration()
        Dim ofd As OpenFileDialog = New OpenFileDialog
        ofd.Filter = "Configuration file (*.txt)|*txt"
        If ofd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            Load_INI(ofd.FileName)
        End If
    End Sub


    Friend Sub Save_INI(Optional ByVal iniFileName As String = "")
        If Form_Pulses Is Nothing Then
            MsgBox("Error - FormPulses is Nothing")
            Return
        End If
        If Form_Equalizers Is Nothing Then
            MsgBox("Error - Form_Equalizers is Nothing")
            Return
        End If

        'If GetFormRectangle(Form_Pulses).Left < -30000 Then
        '    Return
        'End If
        'If GetFormRectangle(Form_Equalizers).Left < -30000 Then
        '    Return
        'End If
        'If GetFormRectangle(Form_Tomy).Left < -30000 Then
        '    Return
        'End If
        '
        Try
            '
            If iniFileName = "" Then
                iniFileName = PlatformAdjustedFileName(Application.StartupPath & "\" & AssemblyName() & "_INI.txt")
            End If
            iniFileName = PlatformAdjustedFileName(iniFileName)
            '
            Dim f As System.IO.StreamWriter
            f = IO.File.CreateText(iniFileName)
            '
            f.WriteLine(" Program Params")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("Language", Language))
            ' ------------------------------------------------------------------------------ FORM BOUNDS
            Dim r As Rectangle
            r = GetFormRectangle(Form_Main)
            f.WriteLine(TabString("Form1_Top", r.Top))
            f.WriteLine(TabString("Form1_Left", r.Left))
            f.WriteLine(TabString("Form1_Width", r.Width))
            f.WriteLine(TabString("Form1_Height", r.Height))
            f.WriteLine(TabString("Form1_WindowState", Form_Main.WindowState))
            '
            r = GetFormRectangle(Form_Pulses)
            f.WriteLine(TabString("Form2_Top", r.Top))
            f.WriteLine(TabString("Form2_Left", r.Left))
            f.WriteLine(TabString("Form2_Width", r.Width))
            f.WriteLine(TabString("Form2_Height", r.Height))
            f.WriteLine(TabString("Form2_WindowState", Form_Pulses.WindowState))
            f.WriteLine(TabString("Form2_Visible", Form_Pulses.Visible))
            '
            r = GetFormRectangle(Form_Equalizers)
            f.WriteLine(TabString("Form3_Top", r.Top))
            f.WriteLine(TabString("Form3_Left", r.Left))
            'f.WriteLine(TabString("Form3_Width", r.Width))
            'f.WriteLine(TabString("Form3_Height", r.Height))
            f.WriteLine(TabString("Form3_WindowState", Form_Equalizers.WindowState))
            f.WriteLine(TabString("Form3_Visible", Form_Equalizers.Visible))
            '
            f.WriteLine("")
            f.WriteLine(" AudioIn")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("Bins", Form_Main.cmb_Bins.SelectedIndex))
            f.WriteLine(TabString("Sampling", Form_Main.cmb_Sampling.SelectedIndex))
            f.WriteLine(TabString("PulsePolarity", Form_Main.cmb_PulsePolarity.SelectedIndex))
            f.WriteLine(TabString("NegPulses", AudioIn.NegPulses.ToString))
            f.WriteLine(TabString("InputDevice", Form_Main.txt_InputDevice.Text))
            f.WriteLine(TabString("AudioGain", Form_Main.txt_AudioGain.Text))
            f.WriteLine(TabString("AudioZero", Form_Main.txt_AudioZero.Text))
            f.WriteLine(TabString("DeadTime", Form_Main.txt_DeadTime.NumericValueInteger))
            '
            f.WriteLine("")
            f.WriteLine("===========================================")
            f.WriteLine(" BaseLine")
            f.WriteLine(TabString("BaseLineEnable", Form_Main.chk_BlrEnable.Checked))
            f.WriteLine(TabString("BaseLinePosition", Form_Main.txt_Blr_Position.NumericValueInteger))
            f.WriteLine(TabString("BaseLineSize", Form_Main.txt_Blr_Size.NumericValueInteger))
            '
            f.WriteLine("")
            f.WriteLine(" Params")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("IntegrationTime", Form_Main.txt_IntegrationTime.NumericValueInteger))
            f.WriteLine(TabString("DrawSpeed", Form_Main.txt_DrawSpeed.NumericValueInteger))
            f.WriteLine(TabString("MinEnergy", Form_Main.txt_MinEnergy.NumericValueInteger))
            f.WriteLine(TabString("IIR_Filter", Form_Main.txt_IIR_Filter.NumericValueInteger))
            '
            f.WriteLine("")
            f.WriteLine(" Export options")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("ExportWithHeader", Form_Main.chk_ExportWithHeader.Checked))
            f.WriteLine(TabString("DecimalSeparator", Form_Main.txt_DecimalSeparator.Text))
            f.WriteLine(TabString("FieldSeparator", Form_Main.txt_FieldSeparator.Text))
            '
            f.WriteLine("")
            f.WriteLine(" Output slots")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("SlotCounter", Form_Main.txt_SlotCounter.Text))
            f.WriteLine(TabString("BinsFirstSlot", Form_Main.txt_BinsFirstSlot.Text))
            f.WriteLine(TabString("BinsNumSlots", Form_Main.txt_BinsNumSlots.Text))
            '
            f.WriteLine("")
            f.WriteLine(" Timers")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("StopTimer", Form_Main.txt_StopTimer.Text))
            '
            f.WriteLine("")
            f.WriteLine(" Scale options")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("XlogExponent", Form_Main.txt_XlogExponent.Text))
            f.WriteLine(TabString("YlogExponent", Form_Main.txt_YlogExponent.Text))
            f.WriteLine(TabString("ThickLines", Form_Main.chk_ThickLines.Checked))
            '
            f.WriteLine("")
            f.WriteLine(" Graph")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("Xlog", Form_Main.btn_Xlog.Checked))
            f.WriteLine(TabString("Ylog", Form_Main.btn_Ylog.Checked))
            f.WriteLine(TabString("Zoom", Form_Main.tk_Zoom.Value))
            f.WriteLine(TabString("LinMaster", Form_Main.tk_LinMaster.Value.ToString))
            f.WriteLine(TabString("EqMaster", Form_Main.tk_EqMaster.Value.ToString))
            '
            f.WriteLine("")
            f.WriteLine(" Form1 - User Text")
            f.WriteLine("===========================================")
            f.WriteLine(MultilineToString("UserText", Form_Main.txt_UserText.Text))
            '
            f.WriteLine("")
            f.WriteLine(" Form2 - ShapeVisualizer")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("ScopeMaxEnergy", Form_Pulses.txt_MaxEnergy.Text))
            f.WriteLine(TabString("ScopeMinEnergy", Form_Pulses.txt_MinEnergy.Text))
            f.WriteLine(TabString("ScopeSizeX", Form_Pulses.tk_SizeX.Value))
            f.WriteLine(TabString("ScopeSizeY", Form_Pulses.tk_SizeY.Value))
            f.WriteLine(TabString("ScopePosY", Form_Pulses.tk_PosY.Value.ToString))
            f.WriteLine(TabString("ScopeRun", Form_Pulses.btn_Run.Checked))
            f.WriteLine(TabString("ScopeNormalize", Form_Pulses.btn_Normalized.Checked))
            '
            f.WriteLine("")
            f.WriteLine(" Form3 - Equalizers")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("LinOn", Form_Equalizers.chk_LinOn.Checked))
            f.WriteLine(TabString("EqOn", Form_Equalizers.chk_EqOn.Checked))
            '
            f.WriteLine(" Tomy")
            f.WriteLine("===========================================")
            r = GetFormRectangle(Form_Tomy)
            f.WriteLine(TabString("FormTomy_Top", r.Top))
            f.WriteLine(TabString("FormTomy_Left", r.Left))
            f.WriteLine(TabString("FormTomy_Width", r.Width))
            f.WriteLine(TabString("FormTomy_Height", r.Height))
            f.WriteLine(TabString("Tomy_Tool_Enabled", Form_Main.ToolStripButton_Identifier.Checked))
            '
            SaveLinearizer(f)
            SaveEqualizer(f)
            '
            f.Close()
        Catch
        End Try
    End Sub

    Friend Sub Load_INI(Optional ByVal iniFileName As String = "")
        ' ----------------------------------------- avoid trackbars and other controls min-max overflow
        On Error Resume Next
        ' 
        Dim LoadAll As Boolean = False
        If iniFileName = "" Then
            iniFileName = Application.StartupPath & "\" & AssemblyName() & "_INI.txt"
            LoadAll = True
        End If
        iniFileName = PlatformAdjustedFileName(iniFileName)
        '
        Dim l As String
        If My.Computer.FileSystem.FileExists(iniFileName) Then
            Dim f As System.IO.StreamReader
            f = IO.File.OpenText(iniFileName)

            Do While Not f.EndOfStream
                l = f.ReadLine()
                Dim param As String = ExtractParamName(l)
                Select Case param
                    Case "Language" : Language = l
                        ' ----------------------------------------------------------------- Form_Main
                    Case "Form1_Top" : If LoadAll Then Form_Main.Top = CInt(Val(l))
                    Case "Form1_Left" : If LoadAll Then Form_Main.Left = CInt(Val(l))
                    Case "Form1_Width" : If LoadAll Then Form_Main.Width = CInt(Val(l))
                    Case "Form1_Height" : If LoadAll Then Form_Main.Height = CInt(Val(l))
                    Case "Form1_WindowState" : If LoadAll Then Form1_StartWindowState = CInt(Val(l))
                        ' ----------------------------------------------------------------- Form Pulses
                    Case "Form2_Top" : If LoadAll Then Form_Pulses.Top = CInt(Val(l))
                    Case "Form2_Left" : If LoadAll Then Form_Pulses.Left = CInt(Val(l))
                    Case "Form2_Width" : If LoadAll Then Form_Pulses.Width = CInt(Val(l))
                    Case "Form2_Height" : If LoadAll Then Form_Pulses.Height = CInt(Val(l))
                    Case "Form2_WindowState" : If LoadAll Then Form2_StartWindowState = CInt(Val(l))
                    Case "Form2_Visible" : If LoadAll Then Form2_StartVisible = l = "True"
                        ' ----------------------------------------------------------------- Form Equalizers
                    Case "Form3_Top" : If LoadAll Then Form_Equalizers.Top = CInt(Val(l))
                    Case "Form3_Left" : If LoadAll Then Form_Equalizers.Left = CInt(Val(l))
                        'Case "Form3_Width" : If LoadAll Then Form_Equalizers.Width = CInt(Val(l))
                        'Case "Form3_Height" : If LoadAll Then Form_Equalizers.Height = CInt(Val(l))
                    Case "Form3_WindowState" : If LoadAll Then Form3_StartWindowState = CInt(Val(l))
                    Case "Form3_Visible" : If LoadAll Then Form3_StartVisible = l = "True"
                        ' ----------------------------------------------------------------- AudioIn
                    Case "Bins" : Form_Main.cmb_Bins.SelectedIndex = CInt(Val(l))
                    Case "Sampling" : Form_Main.cmb_Sampling.SelectedIndex = CInt(Val(l))
                    Case "PulsePolarity" : Form_Main.cmb_PulsePolarity.SelectedIndex = CInt(Val(l))
                    Case "NegPulses" : AudioIn.NegPulses = l = "True"
                    Case "InputDevice" : Form_Main.txt_InputDevice.Text = l
                    Case "AudioGain" : Form_Main.txt_AudioGain.Text = l
                    Case "AudioZero" : Form_Main.txt_AudioZero.Text = l
                    Case "DeadTime" : Form_Main.txt_DeadTime.NumericValueInteger = CInt(Val(l))
                        ' ----------------------------------------------------------------- BaseLine
                    Case "BaseLineEnable" : Form_Main.chk_BlrEnable.Checked = l = "True"
                    Case "BaseLinePosition" : Form_Main.txt_Blr_Position.Text = l
                    Case "BaseLineSize" : Form_Main.txt_Blr_Size.Text = l
                        ' ----------------------------------------------------------------- Params
                    Case "IntegrationTime" : Form_Main.txt_IntegrationTime.NumericValueInteger = CInt(Val(l))
                    Case "DrawSpeed" : Form_Main.txt_DrawSpeed.NumericValueInteger = CInt(Val(l))
                    Case "MinEnergy" : Form_Main.txt_MinEnergy.NumericValueInteger = CInt(Val(l))
                    Case "IIR_Filter" : Form_Main.txt_IIR_Filter.NumericValueInteger = CInt(Val(l))
                        ' ----------------------------------------------------------------- Export options
                    Case "ExportWithHeader" : Form_Main.chk_ExportWithHeader.Checked = l = "True"
                    Case "DecimalSeparator" : Form_Main.txt_DecimalSeparator.Text = l
                    Case "FieldSeparator" : Form_Main.txt_FieldSeparator.Text = l
                        ' ----------------------------------------------------------------- Out slots
                    Case "SlotCounter" : Form_Main.txt_SlotCounter.Text = l
                    Case "BinsFirstSlot" : Form_Main.txt_BinsFirstSlot.Text = l
                    Case "BinsNumSlots" : Form_Main.txt_BinsNumSlots.Text = l
                        ' ----------------------------------------------------------------- Timers
                    Case "StopTimer" : Form_Main.txt_StopTimer.Text = l
                        ' ----------------------------------------------------------------- Scale options
                    Case "XlogExponent" : Form_Main.txt_XlogExponent.Text = l
                    Case "YlogExponent" : Form_Main.txt_YlogExponent.Text = l
                    Case "ThickLines" : Form_Main.chk_ThickLines.Checked = l = "True"
                        ' ----------------------------------------------------------------- Graph
                    Case "Xlog" : Form_Main.btn_Xlog.Checked = l = "True"
                    Case "Ylog" : Form_Main.btn_Ylog.Checked = l = "True"
                    Case "Zoom" : Form_Main.tk_Zoom.Value = CInt(Val(l))
                    Case "LinMaster" : Form_Main.tk_LinMaster.Value = CInt(Val(l))
                    Case "EqMaster" : Form_Main.tk_EqMaster.Value = CInt(Val(l))
                        ' ----------------------------------------------------------------- UserText
                    Case "UserText" : Form_Main.txt_UserText.Text = StringToMultiline(l)
                        ' ----------------------------------------------------------------- Form2 - ShapeVisualizer
                    Case "ScopeMaxEnergy" : Form_Pulses.txt_MaxEnergy.Text = l
                    Case "ScopeMinEnergy" : Form_Pulses.txt_MinEnergy.Text = l
                    Case "ScopeSizeX" : Form_Pulses.tk_SizeX.Value = CInt(Val(l))
                    Case "ScopeSizeY" : Form_Pulses.tk_SizeY.Value = CInt(Val(l))
                    Case "ScopePosY" : Form_Pulses.tk_PosY.Value = CInt(Val(l))
                    Case "ScopeRun" : Form_Pulses.btn_Run.Checked = l = "True"
                    Case "ScopeNormalize" : Form_Pulses.btn_Normalized.Checked = l = "True"
                        ' ----------------------------------------------------------------- Form3 - Equalizers
                    Case "LinOn" : Form_Equalizers.chk_LinOn.Checked = l = "True"
                    Case "EqOn" : Form_Equalizers.chk_EqOn.Checked = l = "True"
                        ' ----------------------------------------------------------------- Tomy
                    Case "FormTomy_Top" : If LoadAll Then Form_Tomy.Top = CInt(Val(l))
                    Case "FormTomy_Left" : If LoadAll Then Form_Tomy.Left = CInt(Val(l))
                    Case "FormTomy_Width" : If LoadAll Then Form_Tomy.Width = CInt(Val(l))
                    Case "FormTomy_Height" : If LoadAll Then Form_Tomy.Height = CInt(Val(l))
                    Case "Tomy_Tool_Enabled" : Form_Main.ToolStripButton_Identifier.Checked = l = "True"
                End Select
                '
                LoadLinearizer(f, param, l)
                LoadEqualizer(f, param, l)
            Loop
            f.Close()
        End If
        If Form_Main.WindowState = FormWindowState.Minimized Then Form_Main.WindowState = FormWindowState.Normal
        LimitFormPosition(Form_Main)
        LimitFormPosition(Form_Pulses)
        LimitFormPosition(Form_Equalizers)
        LimitFormPosition(Form_Tomy)
    End Sub


    ' ==================================================================================================
    '  SAVE LOAD -- LINEARIZER and EQUALIZER
    ' ==================================================================================================
    Friend Sub SaveLinearizerAs()
        Dim sfd As SaveFileDialog = New SaveFileDialog()
        sfd.DefaultExt = ".txt"
        sfd.Filter = "Configuration file (*.txt)|*txt"
        sfd.FileName = "MCA_LINEARIZER_INI" & Date.Now.ToString("yyyy_MM_dd_HH_mm_ss")
        If sfd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            If sfd.FileName = "" Then
                sfd.FileName = PlatformAdjustedFileName(Application.StartupPath & "\" & AssemblyName() & "_LIN_INI.txt")
            End If
            sfd.FileName = PlatformAdjustedFileName(sfd.FileName)
            Dim f As System.IO.StreamWriter
            f = IO.File.CreateText(sfd.FileName)
            SaveLinearizer(f)
            f.Close()
        End If
    End Sub
    Friend Sub LoadLinearizer()
        Dim ofd As OpenFileDialog = New OpenFileDialog
        ofd.Filter = "Configuration file (*.txt)|*txt"
        If ofd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            If ofd.FileName = "" Then
                ofd.FileName = Application.StartupPath & "\" & AssemblyName() & "_LIN_INI.txt"
            End If
            ofd.FileName = PlatformAdjustedFileName(ofd.FileName)
            '
            Dim l As String
            If My.Computer.FileSystem.FileExists(ofd.FileName) Then
                Dim f As System.IO.StreamReader
                f = IO.File.OpenText(ofd.FileName)

                Do While Not f.EndOfStream
                    l = f.ReadLine()
                    Dim param As String = ExtractParamName(l)
                    LoadLinearizer(f, param, l)
                Loop
                f.Close()
            End If
        End If
    End Sub

    Friend Sub SaveEqualizerAs()
        Dim sfd As SaveFileDialog = New SaveFileDialog()
        sfd.DefaultExt = ".txt"
        sfd.Filter = "Configuration file (*.txt)|*txt"
        sfd.FileName = "MCA_EQUALIZER_INI_" & Date.Now.ToString("yyyy_MM_dd_HH_mm_ss")
        If sfd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            If sfd.FileName = "" Then
                sfd.FileName = PlatformAdjustedFileName(Application.StartupPath & "\" & AssemblyName() & "_EQ_INI.txt")
            End If
            sfd.FileName = PlatformAdjustedFileName(sfd.FileName)
            Dim f As System.IO.StreamWriter
            f = IO.File.CreateText(sfd.FileName)
            SaveEqualizer(f)
            f.Close()
        End If
    End Sub
    Friend Sub LoadEqualizer()
        Dim ofd As OpenFileDialog = New OpenFileDialog
        ofd.Filter = "Configuration file (*.txt)|*txt"
        If ofd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            If ofd.FileName = "" Then
                ofd.FileName = Application.StartupPath & "\" & AssemblyName() & "_EQ_INI.txt"
            End If
            ofd.FileName = PlatformAdjustedFileName(ofd.FileName)
            '
            Dim l As String
            If My.Computer.FileSystem.FileExists(ofd.FileName) Then
                Dim f As System.IO.StreamReader
                f = IO.File.OpenText(ofd.FileName)

                Do While Not f.EndOfStream
                    l = f.ReadLine()
                    Dim param As String = ExtractParamName(l)
                    LoadEqualizer(f, param, l)
                Loop
                f.Close()
            End If
        End If
    End Sub

    Private Sub SaveLinearizer(ByVal f As IO.StreamWriter)
        f.WriteLine("")
        f.WriteLine(" Energy Linearizer")
        f.WriteLine("===========================================")
        f.WriteLine(TabString("Lin1", Form_Equalizers.tk_Lin1.Value.ToString))
        f.WriteLine(TabString("Lin2", Form_Equalizers.tk_Lin2.Value.ToString))
        f.WriteLine(TabString("Lin3", Form_Equalizers.tk_Lin3.Value.ToString))
        f.WriteLine(TabString("Lin4", Form_Equalizers.tk_Lin4.Value.ToString))
        f.WriteLine(TabString("Lin5", Form_Equalizers.tk_Lin5.Value.ToString))
        f.WriteLine(TabString("Lin6", Form_Equalizers.tk_Lin6.Value.ToString))
        f.WriteLine(TabString("Lin7", Form_Equalizers.tk_Lin7.Value.ToString))
        f.WriteLine(TabString("Lin8", Form_Equalizers.tk_Lin8.Value.ToString))
        f.WriteLine(TabString("LinInterp", Form_Equalizers.txt_LinInterp.NumericValueInteger))
    End Sub

    Private Sub SaveEqualizer(ByVal f As IO.StreamWriter)
        f.WriteLine("")
        f.WriteLine(" Height Equalizer")
        f.WriteLine("===========================================")
        f.WriteLine(TabString("Eq1", Form_Equalizers.tk_Eq1.Value.ToString))
        f.WriteLine(TabString("Eq2", Form_Equalizers.tk_Eq2.Value.ToString))
        f.WriteLine(TabString("Eq3", Form_Equalizers.tk_Eq3.Value.ToString))
        f.WriteLine(TabString("Eq4", Form_Equalizers.tk_Eq4.Value.ToString))
        f.WriteLine(TabString("Eq5", Form_Equalizers.tk_Eq5.Value.ToString))
        f.WriteLine(TabString("Eq6", Form_Equalizers.tk_Eq6.Value.ToString))
        f.WriteLine(TabString("Eq7", Form_Equalizers.tk_Eq7.Value.ToString))
        f.WriteLine(TabString("Eq8", Form_Equalizers.tk_Eq8.Value.ToString))
        f.WriteLine(TabString("EqInterp", Form_Equalizers.txt_EqInterp.NumericValueInteger))
    End Sub

    Private Sub LoadLinearizer(ByVal f As IO.StreamReader, ByVal param As String, ByVal l As String)
        Select Case param
            Case "Lin1" : Form_Equalizers.tk_Lin1.Value = CInt(Val(l))
            Case "Lin2" : Form_Equalizers.tk_Lin2.Value = CInt(Val(l))
            Case "Lin3" : Form_Equalizers.tk_Lin3.Value = CInt(Val(l))
            Case "Lin4" : Form_Equalizers.tk_Lin4.Value = CInt(Val(l))
            Case "Lin5" : Form_Equalizers.tk_Lin5.Value = CInt(Val(l))
            Case "Lin6" : Form_Equalizers.tk_Lin6.Value = CInt(Val(l))
            Case "Lin7" : Form_Equalizers.tk_Lin7.Value = CInt(Val(l))
            Case "Lin8" : Form_Equalizers.tk_Lin8.Value = CInt(Val(l))
            Case "LinInterp" : Form_Equalizers.txt_LinInterp.NumericValueInteger = CInt(Val(l))
        End Select
    End Sub

    Private Sub LoadEqualizer(ByVal f As IO.StreamReader, ByVal param As String, ByVal l As String)
        Select Case param
            Case "Eq1" : Form_Equalizers.tk_Eq1.Value = CInt(Val(l))
            Case "Eq2" : Form_Equalizers.tk_Eq2.Value = CInt(Val(l))
            Case "Eq3" : Form_Equalizers.tk_Eq3.Value = CInt(Val(l))
            Case "Eq4" : Form_Equalizers.tk_Eq4.Value = CInt(Val(l))
            Case "Eq5" : Form_Equalizers.tk_Eq5.Value = CInt(Val(l))
            Case "Eq6" : Form_Equalizers.tk_Eq6.Value = CInt(Val(l))
            Case "Eq7" : Form_Equalizers.tk_Eq7.Value = CInt(Val(l))
            Case "Eq8" : Form_Equalizers.tk_Eq8.Value = CInt(Val(l))
            Case "EqInterp" : Form_Equalizers.txt_EqInterp.NumericValueInteger = CInt(Val(l))
        End Select
    End Sub


    ' ==================================================================================================
    '  SAVE IMAGE
    ' ==================================================================================================
    Public Sub SaveImage(ByVal img As Image, _
                         ByVal filename As String, _
                         ByVal extension As String, _
                         ByVal Quality As Int32)

        extension = LCase(extension)
        filename = RemoveExtension(filename)
        filename += "." & extension
        '
        If img Is Nothing Then Return
        Try
            File_Kill(filename)
            If extension = "jpg" Then
                Dim ImageEncoders() As ImageCodecInfo = ImageCodecInfo.GetImageEncoders()
                Dim myEncoder As System.Drawing.Imaging.Encoder = System.Drawing.Imaging.Encoder.Quality
                Dim myEncoderParameters As New EncoderParameters(1)
                Dim myEncoderParameter As New EncoderParameter(myEncoder, 100)
                myEncoderParameters.Param(0) = myEncoderParameter
                img.Save(filename, ImageEncoders(1), myEncoderParameters)
            Else
                img.Save(filename, ImageFormatFromFileExtension(extension))
            End If
        Catch
            MsgBox("Image save error", MsgBoxStyle.Exclamation)
        End Try
    End Sub

    Private Function ImageFormatFromFileExtension(ByVal extension As String) As ImageFormat
        Select Case LCase(extension)
            Case "jpg" : Return ImageFormat.Jpeg
            Case "png" : Return ImageFormat.Png
            Case "tiff" : Return ImageFormat.Tiff
            Case "exif" : Return ImageFormat.Exif
            Case "emf" : Return ImageFormat.Emf
            Case "wmf" : Return ImageFormat.Wmf
            Case "gif" : Return ImageFormat.Gif
            Case "bmp" : Return ImageFormat.Bmp
                'Case "ico" : Return ImageFormat.Icon
            Case Else : Return ImageFormat.Jpeg
        End Select
    End Function

    Friend Sub File_Kill(ByVal filename As String)
        If My.Computer.FileSystem.FileExists(filename) Then
            My.Computer.FileSystem.DeleteFile(filename, FileIO.UIOption.OnlyErrorDialogs, FileIO.RecycleOption.DeletePermanently)
        End If
    End Sub


    Friend Sub SaveImage(ByVal Prefix As String, ByVal cnt As Control)
        Dim img As Image = GetImage(cnt)
        Dim sfd As SaveFileDialog = New SaveFileDialog()
        sfd.Filter = "Portable network graphics (*.png)|*png|JPEG image (*.jpg)|*.jpg"
        sfd.FileName = Prefix & Date.Now.ToString("yyyy_MM_dd_HH_mm_ss")
        If sfd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            SaveImage(img, sfd.FileName, If(sfd.FilterIndex = 2, "jpg", "png"), 100)
        End If
    End Sub
    Friend Function GetImage(ByVal cnt As Control) As Bitmap
        Dim s As Size = cnt.Size
        Dim bmp As Bitmap = New Bitmap(s.Width, s.Height)
        cnt.DrawToBitmap(bmp, New Rectangle(0, 0, s.Width, s.Height))
        Return bmp
    End Function



    Friend Sub SaveHistogramFile(ByVal ExportWithHeader As Boolean, _
                                 ByVal DecimalSeparator As String, _
                                 ByVal FieldSeparator As String)
        '
        Dim s As String = Spectrometer.GetBinDataString(ExportWithHeader, DecimalSeparator, FieldSeparator)
        Dim sfd As SaveFileDialog = New SaveFileDialog()
        sfd.Filter = "CSV text file (*.txt)|*.txt|Tutti i file (*.*)|*.*"
        sfd.FileName = "HISTOGRAM_" & Date.Now.ToString("yyyy_MM_dd_HH_mm_ss")
        If sfd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            Dim writer As System.IO.StreamWriter
            writer = New System.IO.StreamWriter(sfd.FileName)
            writer.Write(s)
            writer.Close()
        End If
    End Sub

    Friend Function LoadHistogramFile(ByVal destination As Int32) As Boolean
        Dim ofd As OpenFileDialog = New OpenFileDialog
        ofd.Filter = "Histogram file (*.txt)|*txt"
        If ofd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            Dim StringArray() As String
            StringArray = IO.File.ReadAllLines(ofd.FileName)
            Spectrometer.FillBinsFromStringArray(destination, StringArray)
            Return True
        End If
        Return False
    End Function



End Module
