
Imports System.Runtime.InteropServices


Namespace WaveLib

    Public Enum WaveFormats
        Pcm = 1
        Float = 3
    End Enum

    <StructLayout(LayoutKind.Sequential)> _
    Public Class WaveFormat
        Public wFormatTag As Short
        Public nChannels As Short
        Public nSamplesPerSec As Int32
        Public nAvgBytesPerSec As Int32
        Public nBlockAlign As Short
        Public wBitsPerSample As Short
        Public cbSize As Short

        Public Sub New(ByVal rate As Int32, ByVal bits As Int32, ByVal channels As Int32)
            wFormatTag = CShort(WaveFormats.Pcm)
            nChannels = CShort(channels)
            nSamplesPerSec = rate
            wBitsPerSample = CShort(bits)
            cbSize = 0

            nBlockAlign = CShort((channels * (bits \ 8)))
            nAvgBytesPerSec = nSamplesPerSec * nBlockAlign
        End Sub
    End Class




    ' ========================================================================================================
    '   CLASS - WAVE NATIVE
    ' ========================================================================================================

    Friend Class WaveNative

        ' --------------------------------------------------------------------- consts
        Public Const MMSYSERR_NOERROR As Int32 = 0

        Public Const MM_WOM_OPEN As Int32 = &H3BB
        Public Const MM_WOM_CLOSE As Int32 = &H3BC
        Public Const MM_WOM_DONE As Int32 = &H3BD

        Public Const MM_WIM_OPEN As Int32 = &H3BE
        Public Const MM_WIM_CLOSE As Int32 = &H3BF
        Public Const MM_WIM_DATA As Int32 = &H3C0

        Public Const CALLBACK_FUNCTION As Int32 = &H30000     ' dwCallback is a FARPROC 

        Public Const TIME_MS As Int32 = &H1                   ' time in milliseconds 
        Public Const TIME_SAMPLES As Int32 = &H2              ' number of wave samples 
        Public Const TIME_BYTES As Int32 = &H4                ' current byte offset



        ' --------------------------------------------------------------------- structs 

        <StructLayout(LayoutKind.Sequential)> _
        Public Structure WaveHdr
            Public lpData As IntPtr
            ' pointer to locked data buffer
            Public dwBufferLength As Int32
            ' length of data buffer
            Public dwBytesRecorded As Int32
            ' used for input only
            Public dwUser As IntPtr
            ' for client's use
            Public dwFlags As Int32
            ' assorted flags (see defines)
            Public dwLoops As Int32
            ' loop control counter
            Public lpNext As IntPtr
            ' PWaveHdr, reserved for driver
            Public reserved As Int32
            ' reserved for driver
        End Structure



        ' --------------------------------------------------------------------- callback
        Public Delegate Sub WaveDelegate(ByVal hdrvr As IntPtr, _
                                         ByVal uMsg As Int32, _
                                         ByVal dwUser As Int32, _
                                         ByRef wavhdr As WaveHdr, _
                                         ByVal dwParam2 As Int32)


        ' --------------------------------------------------------------------- WaveOut
        <DllImport("winmm.dll")> _
        Public Shared Function waveOutGetNumDevs() As Int32
        End Function
        <DllImport("winmm.dll")> _
        Public Shared Function waveOutPrepareHeader(ByVal hWaveOut As IntPtr, ByRef lpWaveOutHdr As WaveHdr, ByVal uSize As Int32) As Int32
        End Function
        <DllImport("winmm.dll")> _
        Public Shared Function waveOutUnprepareHeader(ByVal hWaveOut As IntPtr, ByRef lpWaveOutHdr As WaveHdr, ByVal uSize As Int32) As Int32
        End Function
        <DllImport("winmm.dll")> _
        Public Shared Function waveOutWrite(ByVal hWaveOut As IntPtr, ByRef lpWaveOutHdr As WaveHdr, ByVal uSize As Int32) As Int32
        End Function
        <DllImport("winmm.dll")> _
        Public Shared Function waveOutOpen(ByRef hWaveOut As IntPtr, ByVal uDeviceID As Int32, ByVal lpFormat As WaveFormat, ByVal dwCallback As WaveDelegate, ByVal dwInstance As Int32, ByVal dwFlags As Int32) As Int32
        End Function
        <DllImport("winmm.dll")> _
        Public Shared Function waveOutReset(ByVal hWaveOut As IntPtr) As Int32
        End Function
        <DllImport("winmm.dll")> _
        Public Shared Function waveOutClose(ByVal hWaveOut As IntPtr) As Int32
        End Function
        <DllImport("winmm.dll")> _
        Public Shared Function waveOutPause(ByVal hWaveOut As IntPtr) As Int32
        End Function
        <DllImport("winmm.dll")> _
        Public Shared Function waveOutRestart(ByVal hWaveOut As IntPtr) As Int32
        End Function
        <DllImport("winmm.dll")> _
        Public Shared Function waveOutGetPosition(ByVal hWaveOut As IntPtr, ByRef lpInfo As Int32, ByVal uSize As Int32) As Int32
        End Function
        <DllImport("winmm.dll")> _
        Public Shared Function waveOutSetVolume(ByVal hWaveOut As IntPtr, ByVal dwVolume As UInt32) As Int32
        End Function
        <DllImport("winmm.dll")> _
        Public Shared Function waveOutGetVolume(ByVal hWaveOut As IntPtr, ByRef dwVolume As Int32) As Int32
        End Function

        ' --------------------------------------------------------------------- WaveIn
        <DllImport("winmm.dll")> _
        Public Shared Function waveInGetNumDevs() As Int32
        End Function
        <DllImport("winmm.dll")> _
        Public Shared Function waveInAddBuffer(ByVal hwi As IntPtr, ByRef pwh As WaveHdr, ByVal cbwh As Int32) As Int32
        End Function
        <DllImport("winmm.dll")> _
        Public Shared Function waveInClose(ByVal hwi As IntPtr) As Int32
        End Function
        <DllImport("winmm.dll")> _
        Public Shared Function waveInOpen(ByRef phwi As IntPtr, ByVal uDeviceID As Int32, ByVal lpFormat As WaveFormat, ByVal dwCallback As WaveDelegate, ByVal dwInstance As Int32, ByVal dwFlags As Int32) As Int32
        End Function
        <DllImport("winmm.dll")> _
        Public Shared Function waveInPrepareHeader(ByVal hWaveIn As IntPtr, ByRef lpWaveInHdr As WaveHdr, ByVal uSize As Int32) As Int32
        End Function
        <DllImport("winmm.dll")> _
        Public Shared Function waveInUnprepareHeader(ByVal hWaveIn As IntPtr, ByRef lpWaveInHdr As WaveHdr, ByVal uSize As Int32) As Int32
        End Function
        <DllImport("winmm.dll")> _
        Public Shared Function waveInReset(ByVal hwi As IntPtr) As Int32
        End Function
        <DllImport("winmm.dll")> _
        Public Shared Function waveInStart(ByVal hwi As IntPtr) As Int32
        End Function
        <DllImport("winmm.dll")> _
        Public Shared Function waveInStop(ByVal hwi As IntPtr) As Int32
        End Function


        ' -------------------------------------------------------------- volume range = 0 to 65535
        ' New volume setting. The low-order word contains the left-channel volume setting,
        '  and the high-order word contains the right-channel setting. 
        ' A value of 0xFFFF represents full volume, and a value of 0x0000 is silence.
        ' If a device does not support both left and right volume control, 
        '  the low-order word of dwVolume specifies the volume level, and the high-order word is ignored.

        ' If a device identifier is used, then the result of the waveOutSetVolume call applies 
        '  to all instances of the device. 
        ' If a device handle is used, then the result applies only to the instance of the device 
        '  referenced by the device handle.

        '  Using -1 the mixer WaveOut volume is changed
        Public Property WaveOutVolume() As Single
            Get
                Dim v As Int32
                WaveNative.waveOutGetVolume(New IntPtr(-1), v)
                v = v And &HFFFF
                Return v / 65536.0F
            End Get
            Set(ByVal value As Single)
                Dim v As UInt32 = CUInt(value * 65535)
                If v > 65535 Then v = 65535
                v += v << 16
                WaveNative.waveOutSetVolume(New IntPtr(-1), v)
            End Set
        End Property

    End Class

End Namespace
